#include "auth.h"

#include "hash/hash.hpp"

#include <print>

#include <leveldb/db.h>

using leveldb_ptr = leveldb::DB*;

auto user_db    = leveldb_ptr{nullptr};
auto iter_round = 10000;

extern "C"
{
    auto open_user_db() -> int
    {
        auto opts = leveldb::Options{};

        opts.create_if_missing = true;

        auto status = leveldb::DB::Open(opts, "db/user", &user_db);

        if (!status.ok()) {
            std::println(stderr, "Failed to open user database: {}", status.ToString());
            return 0;
        }
        return 1;
    }

    auto close_user_db() -> void
    {
        delete user_db;
    }

    auto check_user_exists(const char* user_id, int* result) -> int
    {
        auto value  = std::string{};
        auto status = user_db->Get(leveldb::ReadOptions{}, user_id, &value);
        if (status.ok()) {
            *result = 1;
        } else if (status.IsNotFound()) {
            *result = 0;
        } else {
            std::println(stderr, "Failed to check user existence: {}", status.ToString());
            return 0;
        }
        return 1;
    }

    auto set_user_password(const char* user_id, const char* password) -> int
    {
        auto status = user_db->Put(leveldb::WriteOptions{}, user_id, generate_hash(password, iter_round));
        if (!status.ok()) {
            std::println(stderr, "Failed to set user password: {}", status.ToString());
            return 0;
        }
        return 1;
    }

    auto login(const char* user_id, const char* password, int* result) -> int
    {
        auto value  = std::string{};
        auto status = user_db->Get(leveldb::ReadOptions{}, user_id, &value);
        if (!status.ok()) {
            std::println(stderr, "Failed to login: {}", status.ToString());
            return 0;
        }
        *result = validate_password(password, value.data());
        return 1;
    }

    auto delete_user(const char* user_id) -> int
    {
        auto write_options = leveldb::WriteOptions{};
        auto status        = user_db->Delete(write_options, user_id);
        return status.ok();
    }
}